!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Extrapolates interface geometry to construct magnetic axis.

!latex \heading{coordinate functions for small $r$}
!latex \item By exapanding $f(x,y)$ in a Taylor series about the origin, $f=\sum f_{i,j} x^i y^j$, assuming the polar coordinate transformation
!latex $x=r\cos\t$ and $y=r\sin\t$, and then using repeated applications of the double angle formulae, we see that 
!latex analytic functions when expressed as functions of the polar coordinates, e.g. $f(r,\theta)=\sum_{m,n} f_{m,n}(r)\cos \theta$, must take the form
!latex \be f_{m,n}(r) = r^m ( a_{m,0} r^0 + a_{m,1} r^2 + a_{m,2} r^4 + a_{m,3} r^6 +\dots ) = \sum_{i=0}^{N-1}f_i r^{m+2i} \label{eq:asymptoticform}
!latex \ee
!latex for small $r$, where $N\equiv$\verb+iextrap+ degrees of freedom are assumed. 

!latex \item In particular, the coordinate functions, $R$ and $Z$, must have this functional form.

!latex \item The degrees of freedom are constrained by fitting, in a least squares sense, to $N$ surfaces, 
!latex \be e = \frac{1}{2} \sum_{j=1}^{N} \left[ f_{m,n}(r_j)-F_j \right]^2,
!latex \ee 
!latex where $F_j$ is the harmonic at the $j$-th surface.

!latex \item The linear equations to be solved are
!latex \be \frac{\partial e}{\partial f_i} = \sum_{j=1}^N \left[ f_{m,n}(r_j) - F_j \right] r^{m+2i} 
!latex                                     = \sum_{j=1}^N \sum_{k=0}^{N-1} f_k r^{m+2k} r^{m+2i} - \sum_{j=1}^N F_j r^{m+2i}=0.
!latex \ee

!latex \item This is cast in standard form, ${\bf A}\cdot{\bf x}={\bf b}$ where $x_k = f_k$,
!latex \be A_{i,k} = \sum_{j=1}^{N} r_j^{m+2k} r_j^{m+2i}, \;\;\;\;\;\;
!latex     b_i = \sum_{j=1}^{N} F_j r_j^{m+2i},\ee
!latex and is solved using the NAG routine \verb+F04ATF+.

!latex \item The coordinate origin is simply determined, by extrapolation, by setting $r=0$ in \Eqn{asymptoticform}.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine ex00aa(lextrap)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  use constants, only : zero
  use numerical, only : vsmall
  use fileunits, only : ounit
  use inputlist, only : Wex00aa, Ni, Ls, Nvol, Ntor
  use cputiming
  use allglobal, only : myid, cpus, ss, Rbc, Zbs, mn, im, in

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS

  INTEGER, intent(in) :: lextrap ! controls number of surfaces = degrees of freedom in asymptotic representation near coordinate axis;

  INTEGER             :: ii, jj, kk, imn, IA, IAA, NN, if04atf, irz, iex, jjmax
  REAL                :: fmn, lss
  REAL                :: Aik(0:lextrap-1,0:lextrap-1), fi(0:lextrap-1), bi(0:lextrap-1) ! matrix, solution and right hand side;
  REAL                :: AA(0:lextrap-1,0:lextrap-1), WKS1(0:lextrap-1), WKS2(0:lextrap-1) ! workspace required for F04ATF;

  BEGIN(ex00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \heading{logical control}

!latex \item From \Eqn{asymptoticform}, only modes with $m=0$ can contribute at the coordinate origin.

!latex \item This routine assumes that the radial coordinate is $s=\sqrt{\psi}$, so an error is encountered if \verb+Ls.ne.2+.
  FATAL(ex00aa,Ls.ne.2)
  
!latex \item The asymptotic form is fit to the interface geometry, so \verb+iextrap+ $\ge 1$ and \verb+iextrap+ $\le$ \verb+Nvol+ is required.
  FATAL(ex00aa,lextrap.lt.1 .or. lextrap.gt.Nvol)

!latex \item This routine is called by \verb+fc00aa+ and \verb+fc01aa+.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( Wex00aa ) then ; jjmax = lextrap ! this includes diagnostic error;
  else               ; jjmax = 0       ! only required to extrapolate coordinate axis;
  end if

  iex = lextrap - 1 ! shorthand

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  Rbc(0,0,1:mn) = zero ; Zbs(0,0,1:mn) = zero ! initialize coordinate axis;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  do imn = 1 , mn ! loop over all harmonics;

   if( im(imn).ne.0 ) cycle
   
   do irz = 1 , 2 ! loop over R,Z;

    if( irz.eq.2 .and. imn.eq.1 ) cycle ! Z_{0,0} sine mode is irrelevant;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
     
    Aik( 0:iex , 0:iex ) = zero ; bi( 0:iex ) = zero ! initialize summation;

    do ii = 0 , iex ! compute matrix, r.h.s.;
     
      do jj = 1 , lextrap
       if( irz.eq.1 ) fmn = Rbc(jj,0,imn)
       if( irz.eq.2 ) fmn = Zbs(jj,0,imn)
       bi(ii) = bi(ii) + fmn * ss(jj)%s(Ni(jj))**( im(imn) + 2*ii ) ! compute r.h.s.;
      enddo
      
      do kk = 0 , iex
       do jj = 1 , lextrap
        Aik(ii,kk) = Aik(ii,kk) + ss(jj)%s(Ni(jj))**( im(imn) + 2*kk + im(imn) + 2*ii ) ! compute matrix;
       enddo
      enddo
      
     enddo ! end of ii = 0,iex;
     
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

     IA = lextrap ; NN = lextrap ; IAA = lextrap ! solve linear system;
     if04atf = 1 ; call F04ATF( Aik(0:iex,0:iex), IA , bi(0:iex) , NN , fi(0:iex) , AA(0:iex,0:iex) , IAA , WKS1(0:iex) , WKS2(0:iex) , if04atf )

     FATALMESS(ex00aa,if04atf.eq.1,error solving for asymptotic form of coordinates near coordinate axis : coordinate extrapolation matrix is singular)
     FATALMESS(ex00aa,if04atf.eq.2,error solving for asymptotic form of coordinates near coordinate axis : coordinate extrapolation matrix is ill-conditioned)
     FATALMESS(ex00aa,if04atf.eq.3,error solving for asymptotic form of coordinates near coordinate axis : input error)

     FATALMESS(ex00aa,if04atf.ne.0,error solving for asymptotic form of coordinates near coordinate axis)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
     
     do jj = 0 , jjmax ! loop over interfaces; assign extrapolated values; compute reconstruction error;
      
      if( jj.eq.0 ) then ; lss = zero
      else               ; lss = ss(jj)%s(Ni(jj))
      endif
      
      fmn = zero ! initialize summation;
      do ii = 0,iex ; fmn = fmn + fi(ii) * lss**( im(imn) + 2*ii ) ! sum expression;
      enddo
      
      if( Wex00aa ) then ! screen output;
       cput = MPI_WTIME()  
       if( jj.eq.0 .and. irz.eq.1 ) write(ounit,1000)cput-cpus,myid,irz,im(imn),in(imn),jj,fmn
       if( jj.eq.0 .and. irz.eq.2 ) write(ounit,1000)cput-cpus,myid,irz,im(imn),in(imn),jj,fmn
       if( jj.gt.0 .and. irz.eq.1 ) write(ounit,1000)cput-cpus,myid,irz,im(imn),in(imn),jj,fmn,Rbc(jj,0,imn),fmn-Rbc(jj,0,imn)
       if( jj.gt.0 .and. irz.eq.2 ) write(ounit,1000)cput-cpus,myid,irz,im(imn),in(imn),jj,fmn,Zbs(jj,0,imn),fmn-Zbs(jj,0,imn)
      endif
      
      if( jj.eq.0 .and. irz.eq.1 ) Rbc(jj,0,imn) = fmn ! this assigns coordinate axis;
      if( jj.eq.0 .and. irz.eq.2 ) Zbs(jj,0,imn) = fmn ! this assigns coordinate axis;
      
      FATALMESS(ex00aa,jj.eq.0 .and. im(imn).ne.0 .and. abs(fmn).gt.vsmall,only the m=0 harmonics can be zero at the origin)
      
     enddo ! end of do jj;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
     
    enddo ! end of do irz;
    
   enddo ! end of do imn;
   
! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! !
   
   if( Wex00aa ) then
    cput = MPI_WTIME()
    write(ounit,'("ex00aa : "f10.2" : myid="i3" ; Raxis="99es23.15)')cput-cpus,myid,Rbc(0,0,1:Ntor+1)
    write(ounit,'("ex00aa : "f10.2" : myid="i3" ; Zaxis="99es23.15)')cput-cpus,myid,Zbs(0,0,1:Ntor+1)
   endif
   
! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! !

  RETURN(ex00aa)

1000 format("ex00aa : "f10.2" : myid="i3" ; irz="i1" ; (m,n)=("i3","i3") : jj="i3" : fmn="es23.15,:,es23.15" ; err="es10.2" ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine ex00aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \heading{comments}
!latex \item The coordinate axis is not required to coincide with the magnetic axis.
